/*
    Eiciel - GNOME editor of ACL file permissions.
    Copyright (C) 2004-2005 Roger Ferrer Ibez

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "eiciel_container.hpp"

#define OAFIID_EICIEL "OAFIID:Eiciel_Component"
#if 0
#define OAFIID_EICIEL_FABRICA "OAFIID:Eiciel_Component_Fabrica"
#endif 
#define PROPERTY_URIS 1


EicielContainer::EicielContainer()
	: escollirFitxer(Gtk::Stock::OPEN), 
	nomFitxer(_("No file opened")),
	sortir(Gtk::Stock::QUIT),
	ajuda(Gtk::Stock::HELP),
	quantA(_("About..."))
{

	set_title("Eiciel");
	set_border_width(4);

	add(principal);

	principal.add(superior);
	superior.add(nomFitxer);
	superior.add(escollirFitxer);

	// CODI FET AMB C
	GtkWidget* control = bonobo_widget_new_control (OAFIID_EICIEL, CORBA_OBJECT_NIL);
	if (!control) 
	{
		g_error (_("Can't create bonobo control. Aborting"));
	}
	else
	{
		gtk_box_pack_start((GtkBox*)(principal.gobj()), control, TRUE, TRUE, 0);
	}

	BonoboControlFrame *control_frame;

	control_frame = bonobo_widget_get_control_frame (BONOBO_WIDGET(control));

	pbc = bonobo_control_frame_get_control_property_bag (control_frame, NULL);

	if (pbc == CORBA_OBJECT_NIL)
		g_error ("Can't connect to property bag. Aborting");
	// FI - CODI FET AMB C

	principal.add(inferior);
	inferior.set_spacing(4);
	inferior.add(sortir);
	inferior.add(ajuda);
	inferior.add(quantA);

	sortir.signal_clicked().connect(
			SigC::slot(*this, &EicielContainer::sortidaAplicacio)
			);

	quantA.signal_clicked().connect (
			SigC::slot(*this, &EicielContainer::mostrarQuantA)
			);

	ajuda.signal_clicked().connect (
			SigC::slot(*this, &EicielContainer::mostrarAjuda)
			);

	escollirFitxer.signal_clicked().connect(SigC::slot(*this, &EicielContainer::oberturaFitxer));

	show_all_children();
}

void EicielContainer::mostrarAjuda()
{
	// Farem com fa l'ajuda de GNOME pero sense usar
	// tota la cutreria de GNOME
	
	// Obtenim la llista de llenguatges. Aix incorporar C en el
	// pitjor dels casos
	const GList *lang_list = gnome_i18n_get_language_list ("LC_MESSAGES");
	Glib::FileTest ftest = Glib::FILE_TEST_EXISTS;

	for (;lang_list != NULL; lang_list = lang_list->next)
	{
		// const char *lang = lang_list->data;
		// Glib::ustring cami_ajuda(lang_list->data);

		// Ho fem comode pero ineficient xD
		Glib::ustring cami_ajuda = Glib::ustring(PKGDATADIR "/doc/") +
			Glib::ustring((const char*)lang_list->data) + 
			Glib::ustring("/eiciel.xml");

		Glib::ustring uri_ajuda = Glib::ustring("ghelp:") + cami_ajuda;
		
		if (Glib::file_test(cami_ajuda, ftest))
		{
			if (!gnome_help_display_uri(uri_ajuda.c_str(), NULL))
			{
				g_warning(_("Could not show the help file even if it was found"));
			}
			else
			{
				g_warning("Hem obert %s", uri_ajuda.c_str());
			}
			return;
		}
	}
	g_warning(_("Help file not found"));
}

void EicielContainer::sortidaAplicacio()
{
	Gtk::Main::quit();
}

void EicielContainer::mostrarQuantA()
{
	EicielAboutBox about(*this);
	about.run();
}

void EicielContainer::oberturaFitxer()
{
    Gtk::FileSelection dialog(_("Choose a file or a directory"));
	dialog.set_transient_for(*this);
	int result = dialog.run();
	dialog.hide();
	switch (result)
	{
		case(Gtk::RESPONSE_OK):
			{
				if (this->obrirFitxer(dialog.get_filename()))
				{
					nomFitxer.set_text(dialog.get_filename());
				}
				else
				{
					Gtk::MessageDialog missatge(_("Could not open the file \"") 
							+ dialog.get_filename() + Glib::ustring("\" ") 
							+ Glib::ustring("(") + missatgeError() + Glib::ustring(")"),
							Gtk::MESSAGE_INFO, Gtk::BUTTONS_OK, true);
					missatge.run();
					nomFitxer.set_text(_("No file opened"));
				}
				break;
			}
		case(Gtk::RESPONSE_CANCEL):
			{
				break;
			}
		default:
			{
				break;
			}
	}
}

Glib::ustring EicielContainer::missatgeError()
{
	CORBA_Environment ev;
	CORBA_exception_init (&ev);

	//return Glib::ustring(bonobo_pbclient_get_string(pbc, "missatgeerror", &ev));
	
	gchar* s = bonobo_pbclient_get_string(pbc, "missatgeerror", &ev);

	return Glib::locale_to_utf8(s);
}

/* Aquest codi est inspirat pel codi de nautilus ^_^U */
bool EicielContainer::obrirFitxer(Glib::ustring nom)
{
	// Primer li passem una llista (d'un sol element)
	// de fitxers
	int i;
	CORBA_sequence_CORBA_string *s;

	// Creem una sequencia corba de string
	s = CORBA_sequence_CORBA_string__alloc ();
	s->_buffer = CORBA_sequence_CORBA_string_allocbuf (1);
	s->_length = 1;

	// Li posem file:///
	char* uri = gnome_vfs_get_uri_from_local_path(nom.c_str());

	// Creem els strings
	for (i = 0; i < 1; i++)
		s->_buffer[i] = CORBA_string_dup (uri);

	// Llibeerem la uri
	free(uri);

	CORBA_Environment ev;

	// Creem un argument de bonobo
	BonoboArg *arg;
	arg = bonobo_arg_new (TC_CORBA_sequence_CORBA_string);
	arg->_value = s;

	// Inicialitzem l'excepcio (molt important!)
	CORBA_exception_init (&ev);
	bonobo_pbclient_set_value (pbc, "uris", arg, &ev);

	// Lliberem l'argument (i tb la sequencia)
	bonobo_arg_release (arg);

	// Ara toca comprovar si s'ha llegit b
	CORBA_exception_init (&ev);
	return bonobo_pbclient_get_boolean(pbc, "fitxerobert", &ev);
}

/* About box */
EicielAboutBox::EicielAboutBox(Gtk::Window& parent)
	: Gtk::Dialog (_("About..."), parent, true, true),
	titol("<span size=\"xx-large\"><b>Eiciel " PACKAGE_VERSION "</b></span>"),
	autor("<small>Copyright © 2004-2005 Roger Ferrer Ibáñez</small>")
{
	set_border_width(4);
	get_vbox()->set_spacing(4);
	titol.set_use_markup();
	autor.set_use_markup();
	get_vbox()->add(titol);
	get_vbox()->add(autor);
	add_button(Gtk::Stock::OK, 0);
	show_all_children();
}
